<?php
/**
 * NOTICE OF LICENSE.
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 * @author    www.mergado.cz
 * @copyright 2016 Mergado technologies, s. r. o.
 * @license   LICENSE.txt
 */

namespace Mergado\Tools;

use DateTime;
use Exception;
use Mergado;
use SimpleXMLElement;


include_once WP_PLUGIN_DIR . '/woocommerce/woocommerce.php';
include_once __MERGADO_DIR__ . 'autoload.php';

class RssClass
{
    const FEED_BASE_URL = 'https://pack.mergado.cz/custom-platform/woocommerce/';

    const FEED_LANGUAGES = ['en', 'cs', 'sk', 'pl', 'hu'];

    const LAST_RSS_FEED_UPDATE = 'mergado_last_rss_feed_download';
    const RSS_FEED_LOCK = 'unfinished_rss_downloads';

    protected $logger;

    public function __construct()
    {
        //Don't add $this->logger = wc_get_logger() .. Jetpack will shout on you
    }

    public function getFeed()
    {
        $now = new DateTime();
        $date = $now->format(NewsClass::DATE_FORMAT);

        try {

            $lastDownload = $this->getLastDownload();

            if ($lastDownload && $lastDownload !== '') {
                $dateTime = new DateTime($lastDownload);
                // Check every half day from last check

                if ($this->getDownloadLock() < count(self::FEED_LANGUAGES) * 3) {
                    $dateFormatted = $dateTime->modify('+5 minutes')->format(NewsClass::DATE_FORMAT);
                } else {
                    $dateFormatted = $dateTime->modify('+30 minutes')->format(NewsClass::DATE_FORMAT);
                }

                if ($dateFormatted <= $date) {
                    foreach (self::FEED_LANGUAGES as $item_lang) {
                        $this->saveFeed($item_lang);
                    }

                    $this->nullDownloadLock();
                    $this->setLastDownload($date);
                }
            } else {
                foreach(self::FEED_LANGUAGES as $item_lang) {
                    $this->saveFeed($item_lang);
                }

                $this->nullDownloadLock();
                $this->setLastDownload($date);
            }
        } catch (Exception $e) {
            wc_get_logger()->warning('Error getting RSS feed. - ' . $e->getMessage());
            $this->increaseDownloadLock();
            $this->setLastDownload($date);
        }
    }

    /**
     * Save new RSS feed articles to database
     *
     * @param $lang
     * @return void
     */
    private function saveFeed($lang)
    {
        try {
            $dbQuery = NewsClass::getNews($lang);
            $rssFeed = $this->downloadFeed($lang);

            foreach ($rssFeed as $item) {

                // Transform keys to lowercase
                $itemAr = (array)$item;
                $item = array_change_key_case($itemAr, CASE_LOWER);

                $itemDatetime = new DateTime((string)$item['pubdate']);
                $save = true;

                if (count($dbQuery) > 0) {
                    foreach ($dbQuery as $dbItem) {

                        // Fix different APIs ( one with time and second only date ) => Compare only based on date and title
                        $dbTime = new DateTime($dbItem->pubDate);
                        $dbTime = $dbTime->format(NewsClass::DATE_COMPARE_FORMAT);

                        if ($itemDatetime->format(NewsClass::DATE_COMPARE_FORMAT) === $dbTime && (string)$item['title'] === $dbItem->title) {
                            $save = false;
                            break;
                        }
                    }
                }

                if ($save) {
                    NewsClass::saveArticle($item, $itemDatetime, $lang);
                }
            }
        } catch (\ParseError $e) {
            wc_get_logger()->warning('Error parsing RSS feed. - ' . $e->getMessage());
        } catch (Exception $e) {
            wc_get_logger()->warning('Error getting RSS feed. - ' . $e->getMessage());
        }
    }

    /**
     * Download RSS feed
     *
     * @param $lang
     * @return array
     */
    private function downloadFeed($lang)
    {
        $lang = NewsClass::getMergadoNewsLanguage($lang);

        $agent = 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.1) Gecko/20061204 Firefox/2.0.0.1';

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_USERAGENT, $agent); //make it act decent
        curl_setopt($ch, CURLOPT_URL, self::FEED_BASE_URL . $lang);
        curl_setopt($ch, CURLOPT_FAILONERROR,1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION,1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); //set this flag for results to the variable
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0); //This is required for HTTPS certs if
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0); //you don't have some key/password action
        $feed = curl_exec($ch);
        curl_close($ch);

        try {
            $x = new SimpleXMLElement($feed, LIBXML_NOERROR);

            $data = array();
            foreach ($x->item as $item) {
                $data[] = $item;
            }
            return $data;

        } catch (Exception $e) {
            throw new \ParseError($e);
        }
    }

    /**
     * Get last download based
     *
     */
    private function getLastDownload()
    {
        return get_option(self::LAST_RSS_FEED_UPDATE, '');
    }

    /**
     * Set last download based
     *
     * @param $now
     */
    private function setLastDownload($now)
    {
        update_option(self::LAST_RSS_FEED_UPDATE, $now);
    }

    /**
     * Set lock for few minutes, if feed is broken
     */
    private function increaseDownloadLock()
    {
        $value = $this->getDownloadLock();
        update_option(self::RSS_FEED_LOCK, $value + 1);
    }

    /**
     * Set download lock to null
     */
    private function nullDownloadLock()
    {
        update_option(self::RSS_FEED_LOCK, 0);
    }

    /**
     * Return current downlaod lock number
     * @return false|string|null
     */
    private function getDownloadLock()
    {
        return get_option(self::RSS_FEED_LOCK, '');
    }
}
